"""Tests for UI improvements."""

from unittest.mock import MagicMock, patch

import pytest
from prompt_toolkit import PromptSession

from agenix.core.messages import (AssistantMessage, MessageUpdateEvent,
                                  ToolExecutionEndEvent,
                                  ToolExecutionStartEvent, TurnEndEvent)
from agenix.ui.cli import CLIRenderer


class TestCLIRenderer:
    """Test CLI renderer improvements."""

    @pytest.fixture
    def renderer(self):
        """Create a CLI renderer."""
        return CLIRenderer()

    def test_renderer_has_prompt_session(self, renderer):
        """Test renderer initializes with prompt_toolkit session."""
        assert hasattr(renderer, 'prompt_session')
        assert isinstance(renderer.prompt_session, PromptSession)

    def test_prompt_uses_prompt_toolkit(self, renderer):
        """Test that prompt() uses prompt_toolkit."""
        # Mock the prompt_session.prompt method
        renderer.prompt_session.prompt = MagicMock(return_value="test input")

        result = renderer.prompt("You")

        assert result == "test input"
        renderer.prompt_session.prompt.assert_called_once()

    def test_prompt_handles_eoferror(self, renderer):
        """Test prompt handles EOFError."""
        renderer.prompt_session.prompt = MagicMock(side_effect=EOFError())

        result = renderer.prompt("You")
        assert result == "/quit"

    def test_prompt_handles_keyboard_interrupt(self, renderer):
        """Test prompt handles KeyboardInterrupt."""
        renderer.prompt_session.prompt = MagicMock(
            side_effect=KeyboardInterrupt())

        result = renderer.prompt("You")
        assert result == "/quit"

    def test_turn_end_no_token_display(self, renderer, capsys):
        """Test that TurnEndEvent doesn't display tokens."""
        # Create event with usage info
        message = AssistantMessage(content="test")
        # Note: usage is None after our fix

        event = TurnEndEvent(message=message, tool_results=[])

        # Render event
        renderer.render_event(event)

        # Capture output
        captured = capsys.readouterr()

        # Should NOT contain token information
        assert "Tokens:" not in captured.out
        assert "tokens" not in captured.out.lower()

    def test_tool_icons(self, renderer):
        """Test that different tools have different icons."""
        tools = ["read", "write", "edit", "bash", "grep"]

        for tool in tools:
            event = ToolExecutionStartEvent(
                tool_name=tool,
                args={"test": "arg"}
            )

            # Should not raise
            renderer.render_event(event)

    def test_message_update_streaming(self, renderer, capsys):
        """Test message streaming output."""
        event = MessageUpdateEvent(message=None, delta="Hello")

        renderer.render_event(event)

        captured = capsys.readouterr()
        assert "Hello" in captured.out


class TestPromptToolkitIntegration:
    """Test prompt_toolkit integration for better unicode support."""

    def test_chinese_input_support(self):
        """Test that prompt_toolkit can handle Chinese characters."""
        renderer = CLIRenderer()

        # Mock to simulate Chinese input
        chinese_text = "你好世界"
        renderer.prompt_session.prompt = MagicMock(return_value=chinese_text)

        result = renderer.prompt("Test")

        assert result == chinese_text
        assert len(result) == 4  # 4 Chinese characters

    def test_prompt_style_configuration(self):
        """Test that prompt has style configuration."""
        renderer = CLIRenderer()

        # Check that style is configured
        assert renderer.prompt_session.style is not None


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



"""Main entry point for agenix."""

import argparse
import asyncio
import os
import sys
from pathlib import Path

from .core.agent import Agent, AgentConfig
from .core.llm import get_provider
from .core.session import SessionManager
from .tools.bash import BashTool
from .tools.edit import EditTool
from .tools.grep import GrepTool
from .tools.read import ReadTool
from .tools.write import WriteTool
from .ui.cli import CLI, CLIRenderer


def parse_args():
    """Parse command line arguments."""
    parser = argparse.ArgumentParser(
        description="Agenix - Lightweight AI coding agent"
    )

    parser.add_argument(
        "--model",
        type=str,
        help="Model to use (e.g., gpt-4o, gpt-4, claude-3-5-sonnet-20241022)"
    )

    parser.add_argument(
        "--api-key",
        type=str,
        help="API key (or set OPENAI_API_KEY env var)"
    )

    parser.add_argument(
        "--base-url",
        type=str,
        help="API base URL (optional, for OpenAI-compatible APIs)"
    )

    parser.add_argument(
        "--working-dir",
        type=str,
        default=".",
        help="Working directory for file operations (default: current directory)"
    )

    parser.add_argument(
        "--system-prompt",
        type=str,
        help="Custom system prompt"
    )

    parser.add_argument(
        "--session",
        type=str,
        help="Session ID to load"
    )

    parser.add_argument(
        "--max-turns",
        type=int,
        default=100,
        help="Maximum conversation turns per prompt (default: 100)"
    )

    parser.add_argument(
        "message",
        nargs="*",
        help="Direct message to process (non-interactive mode)"
    )

    return parser.parse_args()


def get_default_model() -> str:
    """Get default model."""
    return "gpt-4o"


def get_default_system_prompt(tools: list) -> str:
    """Get default system prompt with dynamic guidelines based on available tools.

    Args:
        tools: List of available tool instances
    """
    import datetime

    # Get tool names
    tool_names = {tool.name for tool in tools}

    # Tool descriptions (keep it short - one line per tool)
    tool_descriptions = {
        "read": "Read file contents",
        "write": "Create or overwrite files",
        "edit": "Make surgical edits to files (find exact text and replace)",
        "bash": "Execute bash commands (ls, grep, find, etc.)",
        "grep": "Search file contents for patterns",
    }

    # Build tools list
    tools_list = "\n".join([
        f"- {name}: {tool_descriptions.get(name, 'Tool')}"
        for name in sorted(tool_names)
        if name in tool_descriptions
    ])

    # Build guidelines dynamically based on available tools
    guidelines = []

    has_read = "read" in tool_names
    has_edit = "edit" in tool_names
    has_write = "write" in tool_names
    has_bash = "bash" in tool_names

    # Read before edit guideline
    if has_read and has_edit:
        guidelines.append("Use read to examine files before editing")

    # Edit guideline
    if has_edit:
        guidelines.append("Use edit for precise changes (old text must match exactly)")

    # Write guideline
    if has_write:
        guidelines.append("Use write only for new files or complete rewrites")

    # Output guideline
    if has_edit or has_write:
        guidelines.append("When summarizing your actions, output plain text directly - do NOT use cat or bash to display what you did")

    # Always include these
    guidelines.append("Be concise in your responses")
    guidelines.append("Show file paths clearly when working with files")

    guidelines_text = "\n".join([f"- {g}" for g in guidelines])

    # Get current date/time
    now = datetime.datetime.now()
    date_time = now.strftime("%A, %B %d, %Y at %I:%M:%S %p %Z")

    # Get working directory
    cwd = os.getcwd()

    return f"""You are an expert coding assistant operating inside Agenix, a coding agent harness. You help users by reading files, executing commands, editing code, and writing new files.

Available tools:
{tools_list}

Guidelines:
{guidelines_text}

Current date and time: {date_time}
Current working directory: {cwd}"""


def validate_config(args) -> tuple:
    """Validate configuration and return api_key, base_url, model.

    Returns:
        tuple: (api_key, base_url, model)

    Raises:
        ValueError: If configuration is invalid
    """
    # Get API key
    api_key = args.api_key or os.getenv("OPENAI_API_KEY")
    if not api_key:
        raise ValueError(
            "API key not found. Please set OPENAI_API_KEY environment variable "
            "or use --api-key parameter.\n\n"
            "Example:\n"
            "  export OPENAI_API_KEY='sk-...'\n"
            "  python main.py\n\n"
            "Or:\n"
            "  python main.py --api-key 'sk-...'"
        )

    # Get base URL (optional)
    base_url = args.base_url or os.getenv("OPENAI_BASE_URL")

    # Get model
    model = args.model or get_default_model()

    return api_key, base_url, model


def main():
    """Main entry point."""
    args = parse_args()

    # Setup working directory
    working_dir = os.path.abspath(args.working_dir)
    if not os.path.exists(working_dir):
        print(f"Error: Working directory does not exist: {working_dir}")
        sys.exit(1)

    # Validate configuration
    try:
        api_key, base_url, model = validate_config(args)
    except ValueError as e:
        print(f"Configuration Error: {e}")
        sys.exit(1)

    # Setup tools
    tools = [
        ReadTool(working_dir=working_dir),
        WriteTool(working_dir=working_dir),
        EditTool(working_dir=working_dir),
        BashTool(working_dir=working_dir),
        GrepTool(working_dir=working_dir),
    ]

    # Setup skill directories (in priority order: default -> user -> project)
    skill_dirs = []

    # 1. Default skills (bundled with package, lowest priority)
    default_skills_dir = os.path.join(os.path.dirname(__file__), "default-skills")
    if os.path.exists(default_skills_dir):
        skill_dirs.append(default_skills_dir)

    # 2. User global skills (can override defaults)
    user_skills_dir = os.path.expanduser("~/.agenix/skills")
    if os.path.exists(user_skills_dir):
        skill_dirs.append(user_skills_dir)

    # 3. Project local skills (highest priority, can override both)
    project_skills_dir = os.path.join(working_dir, ".agenix/skills")
    if os.path.exists(project_skills_dir):
        skill_dirs.append(project_skills_dir)

    # Setup agent
    try:
        config = AgentConfig(
            model=model,
            api_key=api_key,
            base_url=base_url,
            system_prompt=args.system_prompt or get_default_system_prompt(tools),
            max_turns=args.max_turns,
            skill_dirs=skill_dirs if skill_dirs else None,
        )
        agent = Agent(config=config, tools=tools)
    except Exception as e:
        print(f"Error initializing agent: {e}")
        sys.exit(1)

    # Setup session management
    session_manager = SessionManager()

    # Load session if specified
    if args.session:
        try:
            messages = session_manager.load_session(args.session)
            agent.messages = messages
            print(f"Loaded session: {args.session} ({len(messages)} messages)")
        except Exception as e:
            print(f"Error loading session: {e}")
            sys.exit(1)

    # Subscribe to agent events for session persistence
    current_session_id = args.session or session_manager.create_session()

    def on_message_end(event):
        """Save messages to session."""
        from agenix.core.messages import MessageEndEvent
        if isinstance(event, MessageEndEvent) and event.message:
            session_manager.save_message(current_session_id, event.message)

    agent.subscribe(on_message_end)

    # Run CLI
    cli = CLI()

    # Check if we have a direct message (non-interactive)
    if args.message:
        message = " ".join(args.message)
        asyncio.run(process_single_message(agent, message, cli.renderer))
    else:
        # Interactive mode
        cli.run_interactive(agent, tools=tools)


async def process_single_message(agent, message: str, renderer: CLIRenderer):
    """Process a single message in non-interactive mode."""
    try:
        renderer.render_message("user", message)

        async for event in agent.prompt(message):
            renderer.render_event(event)

    except Exception as e:
        renderer.render_error(str(e))
        sys.exit(1)


if __name__ == "__main__":
    main()



"""Test configuration for pytest.

This module configures pytest for the Pi-Nano test suite.
"""

import sys
from pathlib import Path

import pytest

# Add project root to Python path
project_root = Path(__file__).parent.parent
sys.path.insert(0, str(project_root))


def pytest_configure(config):
    """Configure pytest with custom markers.

    Args:
        config: Pytest configuration object
    """
    config.addinivalue_line(
        "markers", "asyncio: mark test as async"
    )
    config.addinivalue_line(
        "markers", "slow: mark test as slow running"
    )
    config.addinivalue_line(
        "markers", "integration: mark test as integration test"
    )



"""Test suite for core.session module.

This module contains unit tests for the session management system.
"""

import shutil
import tempfile
from pathlib import Path

import pytest

from agenix.core.messages import (AssistantMessage, TextContent, Usage,
                                  UserMessage)
from agenix.core.session import SessionManager


class TestSessionManager:
    """Test cases for SessionManager class."""

    def setUp(self):
        """Set up test fixtures."""
        self.temp_dir = tempfile.mkdtemp()

    def tearDown(self):
        """Clean up test fixtures."""
        if hasattr(self, 'temp_dir'):
            shutil.rmtree(self.temp_dir)

    def test_create_session_manager(self):
        """Test creating a session manager."""
        self.setUp()
        try:
            manager = SessionManager(session_dir=self.temp_dir)

            assert manager is not None
            assert Path(self.temp_dir).exists()
        finally:
            self.tearDown()

    def test_create_session(self):
        """Test creating a new session."""
        self.setUp()
        try:
            manager = SessionManager(session_dir=self.temp_dir)
            session_id = manager.create_session("test_session")

            assert session_id == "test_session"
            session_file = Path(self.temp_dir) / f"{session_id}.jsonl"
            assert session_file.exists()
        finally:
            self.tearDown()

    def test_create_session_with_auto_id(self):
        """Test creating session with auto-generated ID."""
        self.setUp()
        try:
            manager = SessionManager(session_dir=self.temp_dir)
            session_id = manager.create_session()

            assert session_id is not None
            assert len(session_id) > 0
        finally:
            self.tearDown()

    def test_save_user_message(self):
        """Test saving a user message to session."""
        self.setUp()
        try:
            manager = SessionManager(session_dir=self.temp_dir)
            session_id = manager.create_session("test")

            msg = UserMessage(content="Hello")
            manager.save_message(session_id, msg)

            # Load and verify
            messages = manager.load_session(session_id)
            assert len(messages) == 1
            assert messages[0].content == "Hello"
        finally:
            self.tearDown()

    def test_save_assistant_message(self):
        """Test saving an assistant message to session."""
        self.setUp()
        try:
            manager = SessionManager(session_dir=self.temp_dir)
            session_id = manager.create_session("test")

            usage = Usage(input_tokens=100, output_tokens=50)
            msg = AssistantMessage(
                content=[TextContent(text="Response")],
                model="gpt-4o",
                usage=usage
            )
            manager.save_message(session_id, msg)

            # Load and verify
            messages = manager.load_session(session_id)
            assert len(messages) == 1
            assert messages[0].model == "gpt-4o"
            assert messages[0].usage.input_tokens == 100
        finally:
            self.tearDown()

    def test_load_empty_session(self):
        """Test loading a session with no messages."""
        self.setUp()
        try:
            manager = SessionManager(session_dir=self.temp_dir)
            session_id = manager.create_session("empty")

            messages = manager.load_session(session_id)
            assert len(messages) == 0
        finally:
            self.tearDown()

    def test_load_nonexistent_session(self):
        """Test loading a session that doesn't exist."""
        self.setUp()
        try:
            manager = SessionManager(session_dir=self.temp_dir)

            with pytest.raises(FileNotFoundError):
                manager.load_session("nonexistent")
        finally:
            self.tearDown()

    def test_list_sessions(self):
        """Test listing all sessions."""
        self.setUp()
        try:
            manager = SessionManager(session_dir=self.temp_dir)
            manager.create_session("session1")
            manager.create_session("session2")

            sessions = manager.list_sessions()

            assert len(sessions) == 2
            session_ids = [s["session_id"] for s in sessions]
            assert "session1" in session_ids
            assert "session2" in session_ids
        finally:
            self.tearDown()

    def test_delete_session(self):
        """Test deleting a session."""
        self.setUp()
        try:
            manager = SessionManager(session_dir=self.temp_dir)
            session_id = manager.create_session("delete_me")

            # Verify it exists
            session_file = Path(self.temp_dir) / f"{session_id}.jsonl"
            assert session_file.exists()

            # Delete it
            manager.delete_session(session_id)

            # Verify it's gone
            assert not session_file.exists()
        finally:
            self.tearDown()

    def test_save_and_load_multiple_messages(self):
        """Test saving and loading multiple messages."""
        self.setUp()
        try:
            manager = SessionManager(session_dir=self.temp_dir)
            session_id = manager.create_session("multi")

            # Save multiple messages
            msg1 = UserMessage(content="Hello")
            msg2 = AssistantMessage(content="Hi there", model="gpt-4o")
            msg3 = UserMessage(content="How are you?")

            manager.save_message(session_id, msg1)
            manager.save_message(session_id, msg2)
            manager.save_message(session_id, msg3)

            # Load and verify
            messages = manager.load_session(session_id)

            assert len(messages) == 3
            assert messages[0].role == "user"
            assert messages[1].role == "assistant"
            assert messages[2].role == "user"
        finally:
            self.tearDown()


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



"""Test suite for core.skills module.

This module contains comprehensive unit tests for the Skills management system,
following the Anthropic Agent Skills standard.
"""

import os
import shutil
import tempfile
from pathlib import Path

import pytest

from agenix.core.skills import Skill, SkillManager


class TestSkill:
    """Test cases for Skill dataclass."""

    def test_create_basic_skill(self):
        """Test creating a basic skill with required fields only."""
        skill = Skill(
            name="test-skill",
            description="A test skill",
            file_path="/path/to/skill.md",
            content="# Test Skill\n\nContent here"
        )

        assert skill.name == "test-skill"
        assert skill.description == "A test skill"
        assert skill.file_path == "/path/to/skill.md"
        assert "Test Skill" in skill.content
        assert skill.disable_model_invocation is False
        assert skill.metadata == {}

    def test_skill_with_all_fields(self):
        """Test creating skill with all optional fields."""
        skill = Skill(
            name="test-skill",
            description="A test skill",
            file_path="/path/to/skill.md",
            content="Content",
            license="MIT",
            compatibility="Python 3.8+",
            allowed_tools=["Bash(test:*)", "Read"],
            metadata={"author": "Test Author", "version": "1.0"},
            disable_model_invocation=True
        )

        assert skill.license == "MIT"
        assert skill.compatibility == "Python 3.8+"
        assert skill.allowed_tools == ["Bash(test:*)", "Read"]
        assert skill.metadata["author"] == "Test Author"
        assert skill.metadata["version"] == "1.0"
        assert skill.disable_model_invocation is True

    def test_skill_with_allowed_tools_list(self):
        """Test skill with multiple allowed tools."""
        skill = Skill(
            name="browser",
            description="Browser automation",
            file_path="/path/to/skill.md",
            content="Content",
            allowed_tools=["Bash(browser:*)", "Read", "Write"]
        )

        assert len(skill.allowed_tools) == 3
        assert "Bash(browser:*)" in skill.allowed_tools
        assert "Read" in skill.allowed_tools


class TestSkillManager:
    """Test cases for SkillManager class."""

    def setUp(self):
        """Set up test fixtures."""
        # Create temporary directory
        self.temp_dir = tempfile.mkdtemp()
        self.skills_dir = Path(self.temp_dir) / "skills"
        self.skills_dir.mkdir()

    def tearDown(self):
        """Clean up test fixtures."""
        if hasattr(self, 'temp_dir'):
            shutil.rmtree(self.temp_dir)

    def create_test_skill(self, name, description="Test skill", **kwargs):
        """Helper to create a test skill file with optional fields.

        Args:
            name: Skill name
            description: Skill description
            **kwargs: Additional frontmatter fields (license, allowed-tools, etc.)
        """
        # Build frontmatter
        frontmatter_dict = {
            'name': name,
            'description': description
        }
        frontmatter_dict.update(kwargs)

        # Convert to YAML
        import yaml
        frontmatter_yaml = yaml.dump(
            frontmatter_dict, default_flow_style=False)

        content = f"""---
{frontmatter_yaml}---

# {name.title()} Skill

Test content
"""
        skill_file = self.skills_dir / f"{name}.md"
        skill_file.write_text(content)
        return skill_file

    def test_skill_manager_init(self):
        """Test creating a skill manager."""
        self.setUp()
        try:
            manager = SkillManager(skill_dirs=[str(self.skills_dir)])

            assert manager is not None
            assert len(manager.skill_dirs) >= 1
        finally:
            self.tearDown()

    def test_discover_direct_md_files(self):
        """Test discovering direct .md files in skills directory."""
        self.setUp()
        try:
            # Create test skill
            self.create_test_skill("test-skill")

            manager = SkillManager(skill_dirs=[str(self.skills_dir)])

            assert len(manager.skills) == 1
            assert "test-skill" in manager.skills
        finally:
            self.tearDown()

    def test_discover_skill_md_in_subdirectory(self):
        """Test discovering SKILL.md files in subdirectories."""
        self.setUp()
        try:
            # Create subdirectory with SKILL.md
            skill_subdir = self.skills_dir / "my-skill"
            skill_subdir.mkdir()

            content = """---
name: my-skill
description: A subdirectory skill
---

# My Skill
"""
            (skill_subdir / "SKILL.md").write_text(content)

            manager = SkillManager(skill_dirs=[str(self.skills_dir)])

            assert len(manager.skills) == 1
            assert "my-skill" in manager.skills
        finally:
            self.tearDown()

    def test_validate_skill_name(self):
        """Test skill name validation."""
        manager = SkillManager(skill_dirs=[])

        # Valid names
        assert manager._validate_name("test-skill") is True
        assert manager._validate_name("skill123") is True
        assert manager._validate_name("a") is True

        # Invalid names
        assert manager._validate_name("Test-Skill") is False  # Uppercase
        assert manager._validate_name("-test") is False  # Leading hyphen
        assert manager._validate_name("test-") is False  # Trailing hyphen
        assert manager._validate_name(
            "test--skill") is False  # Consecutive hyphens
        assert manager._validate_name("") is False  # Empty
        assert manager._validate_name("a" * 65) is False  # Too long

    def test_get_skill(self):
        """Test retrieving a skill by name."""
        self.setUp()
        try:
            self.create_test_skill("test-skill")

            manager = SkillManager(skill_dirs=[str(self.skills_dir)])
            skill = manager.get_skill("test-skill")

            assert skill is not None
            assert skill.name == "test-skill"
        finally:
            self.tearDown()

    def test_list_skills(self):
        """Test listing all skills."""
        self.setUp()
        try:
            self.create_test_skill("skill1")
            self.create_test_skill("skill2")

            manager = SkillManager(skill_dirs=[str(self.skills_dir)])
            skills = manager.list_skills()

            assert len(skills) == 2
            skill_names = [s.name for s in skills]
            assert "skill1" in skill_names
            assert "skill2" in skill_names
        finally:
            self.tearDown()

    def test_get_skills_summary(self):
        """Test generating skills summary for system prompt."""
        self.setUp()
        try:
            self.create_test_skill("test-skill", "A test skill for testing")

            manager = SkillManager(skill_dirs=[str(self.skills_dir)])
            summary = manager.get_skills_summary()

            assert "<available_skills>" in summary
            assert 'name="test-skill"' in summary
            assert "A test skill for testing" in summary
            assert "</available_skills>" in summary
        finally:
            self.tearDown()

    def test_disabled_skill_not_in_summary(self):
        """Test that disabled skills don't appear in summary."""
        self.setUp()
        try:
            # Create skill with disable-model-invocation
            content = """---
name: hidden-skill
description: A hidden skill
disable-model-invocation: true
---

# Hidden Skill
"""
            (self.skills_dir / "hidden-skill.md").write_text(content)

            manager = SkillManager(skill_dirs=[str(self.skills_dir)])
            summary = manager.get_skills_summary()

            # Summary should be empty or not contain the hidden skill
            assert "hidden-skill" not in summary
        finally:
            self.tearDown()

    def test_load_skill_content(self):
        """Test loading full skill content."""
        self.setUp()
        try:
            self.create_test_skill("test-skill")

            manager = SkillManager(skill_dirs=[str(self.skills_dir)])
            content = manager.load_skill_content("test-skill")

            assert content is not None
            assert "Test-Skill Skill" in content or "test-skill" in content
        finally:
            self.tearDown()

    def test_name_collision_warning(self):
        """Test that name collisions are detected."""
        self.setUp()
        try:
            # Create two skills with same name in different locations
            self.create_test_skill("duplicate")

            # Create second directory
            skills_dir2 = Path(self.temp_dir) / "skills2"
            skills_dir2.mkdir()

            content = """---
name: duplicate
description: Another duplicate
---

# Duplicate
"""
            (skills_dir2 / "duplicate.md").write_text(content)

            # Load from both directories
            manager = SkillManager(
                skill_dirs=[str(self.skills_dir), str(skills_dir2)])

            # Should only have one skill (first one found)
            assert len(manager.skills) == 1
            assert "duplicate" in manager.skills
        finally:
            self.tearDown()

    def test_add_skill_dir(self):
        """Test adding a skill directory dynamically."""
        self.setUp()
        try:
            self.create_test_skill("skill1")

            # Create manager with first directory
            manager = SkillManager(skill_dirs=[str(self.skills_dir)])
            assert len(manager.skills) == 1

            # Create second directory with another skill
            skills_dir2 = Path(self.temp_dir) / "skills2"
            skills_dir2.mkdir()

            content = """---
name: skill2
description: Second skill
---

# Skill 2
"""
            (skills_dir2 / "skill2.md").write_text(content)

            # Add second directory
            manager.add_skill_dir(str(skills_dir2))

            assert len(manager.skills) == 2
            assert "skill2" in manager.skills
        finally:
            self.tearDown()

    def test_skill_with_allowed_tools_string(self):
        """Test parsing allowed-tools as a single string."""
        self.setUp()
        try:
            content = """---
name: browser
description: Browser automation
allowed-tools: Bash(browser:*)
---

# Browser Skill
"""
            (self.skills_dir / "browser.md").write_text(content)

            manager = SkillManager(skill_dirs=[str(self.skills_dir)])
            skill = manager.get_skill("browser")

            assert skill is not None
            assert skill.allowed_tools == ["Bash(browser:*)"]
        finally:
            self.tearDown()

    def test_skill_with_allowed_tools_list(self):
        """Test parsing allowed-tools as a list."""
        self.setUp()
        try:
            self.create_test_skill(
                "multi-tool",
                description="Multi-tool skill",
                **{"allowed-tools": ["Bash(tool:*)", "Read", "Write"]}
            )

            manager = SkillManager(skill_dirs=[str(self.skills_dir)])
            skill = manager.get_skill("multi-tool")

            assert skill is not None
            assert len(skill.allowed_tools) == 3
            assert "Bash(tool:*)" in skill.allowed_tools
            assert "Read" in skill.allowed_tools
            assert "Write" in skill.allowed_tools
        finally:
            self.tearDown()

    def test_skill_with_all_optional_fields(self):
        """Test skill with all optional Anthropic spec fields."""
        self.setUp()
        try:
            self.create_test_skill(
                "full-skill",
                description="Complete skill with all fields",
                license="MIT",
                compatibility="Python 3.8+",
                **{
                    "allowed-tools": ["Bash(full:*)", "Read"],
                    "disable-model-invocation": True,
                    "metadata": {"author": "Test", "version": "1.0"}
                }
            )

            manager = SkillManager(skill_dirs=[str(self.skills_dir)])
            skill = manager.get_skill("full-skill")

            assert skill is not None
            assert skill.license == "MIT"
            assert skill.compatibility == "Python 3.8+"
            assert skill.allowed_tools == ["Bash(full:*)", "Read"]
            assert skill.disable_model_invocation is True
            assert skill.metadata["author"] == "Test"
            assert skill.metadata["version"] == "1.0"
        finally:
            self.tearDown()

    def test_parse_frontmatter_with_allowed_tools(self):
        """Test frontmatter parsing with allowed-tools field."""
        manager = SkillManager(skill_dirs=[])

        content = """---
name: test
description: Test skill
allowed-tools:
  - Bash(test:*)
  - Read
---

# Content
"""
        frontmatter = manager._parse_frontmatter(content)

        assert frontmatter is not None
        assert "allowed-tools" in frontmatter
        assert isinstance(frontmatter["allowed-tools"], list)
        assert len(frontmatter["allowed-tools"]) == 2


class TestDefaultSkills:
    """Test cases for loading bundled default skills."""

    def test_load_pdf_skill(self):
        """Test loading the pdf skill from default-skills."""
        # Use default-skills directory (bundled with package)
        default_skills_dir = os.path.join(
            os.path.dirname(__file__), "..", "..", "agenix", "default-skills")

        manager = SkillManager(skill_dirs=[default_skills_dir])
        skill = manager.get_skill("pdf")

        assert skill is not None
        assert skill.name == "pdf"
        assert "PDF" in skill.description or "pdf" in skill.description
        assert skill.license is not None
        assert "SKILL.md" in skill.file_path
        assert "default-skills" in skill.file_path

    def test_load_xlsx_skill(self):
        """Test loading the xlsx skill from default-skills."""
        default_skills_dir = os.path.join(
            os.path.dirname(__file__), "..", "..", "agenix", "default-skills")

        manager = SkillManager(skill_dirs=[default_skills_dir])
        skill = manager.get_skill("xlsx")

        assert skill is not None
        assert skill.name == "xlsx"
        assert "spreadsheet" in skill.description.lower(
        ) or "xlsx" in skill.description.lower()
        assert skill.license is not None
        assert "default-skills" in skill.file_path

    def test_load_browser_use_skill(self):
        """Test loading the browser-use skill with allowed-tools."""
        default_skills_dir = os.path.join(
            os.path.dirname(__file__), "..", "..", "agenix", "default-skills")

        manager = SkillManager(skill_dirs=[default_skills_dir])
        skill = manager.get_skill("browser-use")

        assert skill is not None
        assert skill.name == "browser-use"
        assert "browser" in skill.description.lower()
        # Check that allowed-tools is parsed
        assert skill.allowed_tools is not None
        assert len(skill.allowed_tools) > 0
        assert any("browser" in tool.lower() for tool in skill.allowed_tools)
        assert "default-skills" in skill.file_path

    def test_load_all_default_skills(self):
        """Test loading all default skills."""
        default_skills_dir = os.path.join(
            os.path.dirname(__file__), "..", "..", "agenix", "default-skills")

        manager = SkillManager(skill_dirs=[default_skills_dir])
        skills = manager.list_skills()

        # Should have the 7 default skills
        assert len(skills) >= 7

        # Check that all skills have required fields
        for skill in skills:
            assert skill.name
            assert skill.description
            assert skill.file_path
            assert skill.content
            assert len(skill.name) <= 64
            assert skill.name.islower() or '-' in skill.name
            assert "default-skills" in skill.file_path

        # Check for expected default skills
        skill_names = {skill.name for skill in skills}
        expected_skills = {"pdf", "xlsx", "docx", "pptx", "browser-use", "find-skills", "skill-creator"}
        assert expected_skills.issubset(skill_names), f"Missing skills: {expected_skills - skill_names}"

    def test_skills_summary_excludes_disabled(self):
        """Test that skills summary excludes disabled skills."""
        default_skills_dir = os.path.join(
            os.path.dirname(__file__), "..", "..", "agenix", "default-skills")

        manager = SkillManager(skill_dirs=[default_skills_dir])
        summary = manager.get_skills_summary()

        # Summary should have available_skills tag
        assert "<available_skills>" in summary
        assert "</available_skills>" in summary

        # Check that visible skills are included
        for skill in manager.list_skills():
            if not skill.disable_model_invocation:
                assert f'name="{skill.name}"' in summary


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



"""Tests for Skills integration in Agent."""

import os

import pytest

from agenix.core.agent import Agent, AgentConfig

# Get path to default skills for testing
DEFAULT_SKILLS_PATH = os.path.join(
    os.path.dirname(os.path.dirname(os.path.dirname(__file__))),
    "agenix/default-skills"
)


class TestSkillsIntegration:
    """Test Skills system integration with Agent."""

    def test_skills_injected_into_system_prompt(self):
        """Test that skills are properly injected into system prompt."""
        config = AgentConfig(
            api_key="test-key",
            base_url="https://api.example.com/v1",
            model="gpt-4",
            system_prompt="Test prompt.",
            skill_dirs=[DEFAULT_SKILLS_PATH]
        )

        agent = Agent(config)

        # Check that system prompt was modified
        assert config.system_prompt != "Test prompt."
        assert "<available_skills>" in config.system_prompt
        assert "</available_skills>" in config.system_prompt

    def test_skills_usage_instructions_included(self):
        """Test that usage instructions are included."""
        config = AgentConfig(
            api_key="test-key",
            base_url="https://api.example.com/v1",
            model="gpt-4",
            system_prompt="Test.",
            skill_dirs=[DEFAULT_SKILLS_PATH]
        )

        agent = Agent(config)
        prompt = config.system_prompt

        # Check for simple pi-mono style instructions
        assert "The following skills provide specialized instructions" in prompt
        assert "Use the read tool to load a skill's file" in prompt
        assert "When a skill file references a relative path" in prompt

    def test_skill_file_paths_included(self):
        """Test that skill file paths are included in XML format."""
        config = AgentConfig(
            api_key="test-key",
            base_url="https://api.example.com/v1",
            model="gpt-4",
            system_prompt="Test.",
            skill_dirs=[DEFAULT_SKILLS_PATH]
        )

        agent = Agent(config)
        prompt = config.system_prompt

        # Check that skill paths are included in <location> tags
        assert "<location>" in prompt
        assert "</location>" in prompt
        assert "/default-skills/pdf/SKILL.md</location>" in prompt
        assert "/default-skills/browser-use/SKILL.md</location>" in prompt

    def test_xml_format_structure(self):
        """Test that skills use proper XML format."""
        config = AgentConfig(
            api_key="test-key",
            base_url="https://api.example.com/v1",
            model="gpt-4",
            system_prompt="Test.",
            skill_dirs=[DEFAULT_SKILLS_PATH]
        )

        agent = Agent(config)
        prompt = config.system_prompt

        # Check XML structure
        assert "<skill>" in prompt
        assert "</skill>" in prompt
        assert "<name>" in prompt
        assert "</name>" in prompt
        assert "<description>" in prompt
        assert "</description>" in prompt
        assert "<location>" in prompt
        assert "</location>" in prompt

    def test_no_skills_directory(self):
        """Test behavior when no skills directory exists."""
        config = AgentConfig(
            api_key="test-key",
            base_url="https://api.example.com/v1",
            model="gpt-4",
            system_prompt="Test prompt.",
            skill_dirs=["/nonexistent/path"]
        )

        agent = Agent(config)

        # System prompt should not be modified if no skills found
        assert config.system_prompt == "Test prompt."

    def test_skill_manager_loaded(self):
        """Test that SkillManager is properly initialized."""
        config = AgentConfig(
            api_key="test-key",
            base_url="https://api.example.com/v1",
            model="gpt-4",
            skill_dirs=[DEFAULT_SKILLS_PATH]
        )

        agent = Agent(config)

        # Check skill manager exists
        assert agent.skill_manager is not None
        assert len(agent.skill_manager.list_skills()) > 0

        # Check specific skills are loaded
        skill_names = [s.name for s in agent.skill_manager.list_skills()]
        assert "pdf" in skill_names
        assert "browser-use" in skill_names


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



"""Tests for the SDK (programmatic API)."""

import asyncio
import os
from unittest.mock import AsyncMock, MagicMock, patch

import pytest

from agenix.core.messages import MessageEndEvent, MessageUpdateEvent
from agenix.sdk import AgentSession, create_session


class TestSDK:
    """Test SDK functionality."""

    @pytest.mark.asyncio
    async def test_create_session_with_api_key(self):
        """Test creating a session with explicit API key."""
        session = await create_session(
            api_key="test-key",
            model="gpt-4o",
            enable_extensions=False  # Disable for simpler test
        )

        assert session is not None
        assert isinstance(session, AgentSession)
        assert session.agent is not None

    @pytest.mark.asyncio
    async def test_create_session_from_env(self, monkeypatch):
        """Test creating a session using environment variable."""
        monkeypatch.setenv("OPENAI_API_KEY", "env-test-key")

        session = await create_session(
            model="gpt-4o",
            enable_extensions=False
        )

        assert session is not None

    @pytest.mark.asyncio
    async def test_create_session_no_api_key(self, monkeypatch):
        """Test that session creation fails without API key."""
        monkeypatch.delenv("OPENAI_API_KEY", raising=False)

        with pytest.raises(ValueError, match="API key required"):
            await create_session(enable_extensions=False)

    @pytest.mark.asyncio
    async def test_session_get_messages(self):
        """Test getting conversation messages."""
        session = await create_session(
            api_key="test-key",
            model="gpt-4o",
            enable_extensions=False
        )

        # Initially empty
        messages = session.get_messages()
        assert messages == []

    @pytest.mark.asyncio
    async def test_session_clear_messages(self):
        """Test clearing conversation messages."""
        session = await create_session(
            api_key="test-key",
            model="gpt-4o",
            enable_extensions=False
        )

        # Add some mock messages
        from agenix.core.messages import UserMessage
        session.agent.messages.append(UserMessage(content="test"))

        # Clear
        session.clear_messages()

        # Verify cleared
        assert len(session.get_messages()) == 0

    @pytest.mark.asyncio
    async def test_session_working_dir(self):
        """Test session respects working directory."""
        import tempfile

        with tempfile.TemporaryDirectory() as tmpdir:
            session = await create_session(
                api_key="test-key",
                model="gpt-4o",
                working_dir=tmpdir,
                enable_extensions=False
            )

            assert session.working_dir == tmpdir

    @pytest.mark.asyncio
    async def test_session_close(self):
        """Test session cleanup."""
        session = await create_session(
            api_key="test-key",
            model="gpt-4o",
            enable_extensions=False
        )

        # Should not raise
        await session.close()

    @pytest.mark.asyncio
    async def test_session_with_extensions(self, monkeypatch):
        """Test session with extensions enabled."""
        # Mock extension loading to avoid filesystem dependencies
        async def mock_load_extensions(cwd):
            return []

        with patch('agenix.sdk.discover_and_load_extensions', new=mock_load_extensions):
            session = await create_session(
                api_key="test-key",
                model="gpt-4o",
                enable_extensions=True
            )

            assert session is not None


class TestAgentSession:
    """Test AgentSession class."""

    @pytest.fixture
    def mock_agent(self):
        """Create a mock agent."""
        agent = MagicMock()
        agent.messages = []
        agent.clear_messages = MagicMock()
        return agent

    @pytest.fixture
    def session(self, mock_agent):
        """Create a test session."""
        return AgentSession(
            agent=mock_agent,
            tools=[],
            extension_runner=None,
            working_dir="/tmp"
        )

    @pytest.mark.asyncio
    async def test_prompt_collects_response(self, session, mock_agent):
        """Test that prompt() collects the response text."""
        # Mock the agent.prompt to return events
        async def mock_prompt(message):
            yield MessageUpdateEvent(message=None, delta="Hello ")
            yield MessageUpdateEvent(message=None, delta="world!")
            yield MessageEndEvent(message=None)

        mock_agent.prompt = mock_prompt

        response = await session.prompt("test")
        assert response == "Hello world!"

    def test_get_messages(self, session, mock_agent):
        """Test get_messages returns agent messages."""
        from agenix.core.messages import UserMessage

        mock_agent.messages = [UserMessage(content="test")]

        messages = session.get_messages()
        assert len(messages) == 1
        assert messages[0].content == "test"

    def test_clear_messages(self, session, mock_agent):
        """Test clear_messages calls agent method."""
        session.clear_messages()
        mock_agent.clear_messages.assert_called_once()


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



"""Test suite for tools.edit module.

This module contains unit tests for the edit file tool.
"""

import shutil
import tempfile
from pathlib import Path

import pytest

from agenix.tools.edit import EditTool


class TestEditTool:
    """Test cases for EditTool class."""

    def setUp(self):
        """Set up test fixtures."""
        self.temp_dir = tempfile.mkdtemp()

    def tearDown(self):
        """Clean up test fixtures."""
        if hasattr(self, 'temp_dir'):
            shutil.rmtree(self.temp_dir)

    def test_create_edit_tool(self):
        """Test creating an edit tool instance."""
        tool = EditTool(working_dir=".")

        assert tool.name == "edit"
        assert "Edit" in tool.description
        assert "file_path" in tool.parameters["properties"]
        assert "old_string" in tool.parameters["properties"]
        assert "new_string" in tool.parameters["properties"]

    @pytest.mark.asyncio
    async def test_simple_edit(self):
        """Test simple string replacement."""
        self.setUp()
        try:
            # Create test file
            test_file = Path(self.temp_dir) / "test.py"
            test_file.write_text("def old_function():\n    pass\n")

            tool = EditTool(working_dir=self.temp_dir)
            result = await tool.execute(
                tool_call_id="call_123",
                arguments={
                    "file_path": "test.py",
                    "old_string": "old_function",
                    "new_string": "new_function"
                }
            )

            assert result.is_error is False
            assert "Successfully replaced" in result.content

            # Verify edit
            content = test_file.read_text()
            assert "new_function" in content
            assert "old_function" not in content
        finally:
            self.tearDown()

    @pytest.mark.asyncio
    async def test_edit_preserves_whitespace(self):
        """Test that edit preserves exact whitespace."""
        self.setUp()
        try:
            test_file = Path(self.temp_dir) / "test.py"
            original = "def function():\n    return 42\n"
            test_file.write_text(original)

            tool = EditTool(working_dir=self.temp_dir)
            result = await tool.execute(
                tool_call_id="call_123",
                arguments={
                    "file_path": "test.py",
                    "old_string": "    return 42",
                    "new_string": "    return 100"
                }
            )

            assert result.is_error is False
            content = test_file.read_text()
            assert "return 100" in content
        finally:
            self.tearDown()

    @pytest.mark.asyncio
    async def test_edit_generates_diff(self):
        """Test that edit generates a diff."""
        self.setUp()
        try:
            test_file = Path(self.temp_dir) / "test.txt"
            test_file.write_text("Hello World")

            tool = EditTool(working_dir=self.temp_dir)
            result = await tool.execute(
                tool_call_id="call_123",
                arguments={
                    "file_path": "test.txt",
                    "old_string": "World",
                    "new_string": "Python"
                }
            )

            # Check that details include diff
            assert result.details is not None
            assert "diff" in result.details
        finally:
            self.tearDown()

    @pytest.mark.asyncio
    async def test_edit_string_not_found(self):
        """Test edit when old_string is not found."""
        self.setUp()
        try:
            test_file = Path(self.temp_dir) / "test.txt"
            test_file.write_text("Hello World")

            tool = EditTool(working_dir=self.temp_dir)
            result = await tool.execute(
                tool_call_id="call_123",
                arguments={
                    "file_path": "test.txt",
                    "old_string": "Goodbye",
                    "new_string": "Hello"
                }
            )

            assert result.is_error is True
            assert "not found" in result.content.lower()
        finally:
            self.tearDown()

    @pytest.mark.asyncio
    async def test_edit_replace_all(self):
        """Test replacing all occurrences."""
        self.setUp()
        try:
            test_file = Path(self.temp_dir) / "test.txt"
            test_file.write_text("foo bar foo baz foo")

            tool = EditTool(working_dir=self.temp_dir)
            result = await tool.execute(
                tool_call_id="call_123",
                arguments={
                    "file_path": "test.txt",
                    "old_string": "foo",
                    "new_string": "qux",
                    "replace_all": True
                }
            )

            assert result.is_error is False
            content = test_file.read_text()
            assert content.count("qux") == 3
            assert "foo" not in content
        finally:
            self.tearDown()

    @pytest.mark.asyncio
    async def test_edit_replace_first_only(self):
        """Test replacing only first occurrence."""
        self.setUp()
        try:
            test_file = Path(self.temp_dir) / "test.txt"
            test_file.write_text("foo bar foo baz")

            tool = EditTool(working_dir=self.temp_dir)
            result = await tool.execute(
                tool_call_id="call_123",
                arguments={
                    "file_path": "test.txt",
                    "old_string": "foo",
                    "new_string": "qux",
                    "replace_all": False
                }
            )

            assert result.is_error is False
            content = test_file.read_text()
            # Should have one qux and one remaining foo
            assert content.count("qux") == 1
            assert content.count("foo") == 1
        finally:
            self.tearDown()


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



"""Agenix - A lightweight AI coding agent.

A minimal Python implementation inspired by pi-mono, supporting OpenAI and
Anthropic LLMs with file operations, shell execution, Agent Skills, and Extensions.

## Programmatic Usage (SDK)

```python
import asyncio
from agenix import create_session

async def main():
    session = await create_session(
        api_key="your-key",
        model="gpt-4o"
    )
    response = await session.prompt("What files are in the current directory?")
    print(response)

asyncio.run(main())
```

## Extension System

Extensions can be placed in:
- Global: `~/.agenix/extensions/`
- Project: `.agenix/extensions/`

Example extension:
```python
# ~/.agenix/extensions/my_tool.py
async def setup(agenix):
    from agenix.extensions import ToolDefinition, EventType

    async def my_execute(params, ctx):
        return f"Result: {params}"

    agenix.register_tool(ToolDefinition(
        name="my_tool",
        description="My custom tool",
        parameters={"type": "object"},
        execute=my_execute
    ))
```
"""

__version__ = "0.0.1"
__author__ = "ZHANG Zhi"
__email__ = "tczhangzhi@gmail.com"
__license__ = "MIT"

# Core
from .core.agent import Agent, AgentConfig
from .core.llm import get_provider
from .core.session import SessionManager
from .core.skills import Skill, SkillManager
from .core.messages import (
    Message,
    UserMessage,
    AssistantMessage,
    ToolResultMessage,
    TextContent,
    ImageContent,
    ToolCall,
)

# Tools
from .tools.read import ReadTool
from .tools.write import WriteTool
from .tools.edit import EditTool
from .tools.bash import BashTool
from .tools.grep import GrepTool

# SDK
from .sdk import create_session, AgentSession

# Extensions (for extension authors)
from . import extensions

__all__ = [
    # SDK (main API for programmatic usage)
    "create_session",
    "AgentSession",

    # Core
    "Agent",
    "AgentConfig",
    "get_provider",
    "SessionManager",
    "Skill",
    "SkillManager",

    # Messages
    "Message",
    "UserMessage",
    "AssistantMessage",
    "ToolResultMessage",
    "TextContent",
    "ImageContent",
    "ToolCall",

    # Tools
    "ReadTool",
    "WriteTool",
    "EditTool",
    "BashTool",
    "GrepTool",

    # Extensions
    "extensions",
]



"""Test suite for tools.grep module.

This module contains unit tests for the grep search tool.
"""

import shutil
import tempfile
from pathlib import Path

import pytest

from agenix.tools.grep import GrepTool


class TestGrepTool:
    """Test cases for GrepTool class."""

    def setUp(self):
        """Set up test fixtures."""
        self.temp_dir = tempfile.mkdtemp()

    def tearDown(self):
        """Clean up test fixtures."""
        if hasattr(self, 'temp_dir'):
            shutil.rmtree(self.temp_dir)

    def test_create_grep_tool(self):
        """Test creating a grep tool instance."""
        tool = GrepTool(working_dir=".")

        assert tool.name == "grep"
        assert "Search" in tool.description or "search" in tool.description
        assert "pattern" in tool.parameters["properties"]

    @pytest.mark.asyncio
    async def test_simple_search(self):
        """Test simple pattern search."""
        self.setUp()
        try:
            # Create test files
            (Path(self.temp_dir) / "test1.txt").write_text("Hello World\nGoodbye")
            (Path(self.temp_dir) / "test2.txt").write_text("Hello Python")

            tool = GrepTool(working_dir=self.temp_dir)
            result = await tool.execute(
                tool_call_id="call_123",
                arguments={"pattern": "Hello"}
            )

            assert result.is_error is False
            assert "Hello" in result.content
            # Should find matches in both files
            assert "test1.txt" in result.content or "test2.txt" in result.content
        finally:
            self.tearDown()

    @pytest.mark.asyncio
    async def test_regex_search(self):
        """Test regex pattern search."""
        self.setUp()
        try:
            (Path(self.temp_dir) /
             "test.py").write_text("def function1():\ndef function2():\n")

            tool = GrepTool(working_dir=self.temp_dir)
            result = await tool.execute(
                tool_call_id="call_123",
                arguments={"pattern": r"def \w+\(\):"}
            )

            assert result.is_error is False
            assert "function1" in result.content or "function2" in result.content
        finally:
            self.tearDown()

    @pytest.mark.asyncio
    async def test_file_pattern_filter(self):
        """Test filtering by file pattern."""
        self.setUp()
        try:
            (Path(self.temp_dir) / "test.py").write_text("import os")
            (Path(self.temp_dir) / "test.txt").write_text("import os")

            tool = GrepTool(working_dir=self.temp_dir)
            result = await tool.execute(
                tool_call_id="call_123",
                arguments={
                    "pattern": "import",
                    "file_pattern": "*.py"
                }
            )

            assert result.is_error is False
            assert "test.py" in result.content
            # Should not include .txt file
            assert "test.txt" not in result.content
        finally:
            self.tearDown()

    @pytest.mark.asyncio
    async def test_case_insensitive_search(self):
        """Test case insensitive search."""
        self.setUp()
        try:
            (Path(self.temp_dir) / "test.txt").write_text("Hello WORLD")

            tool = GrepTool(working_dir=self.temp_dir)
            result = await tool.execute(
                tool_call_id="call_123",
                arguments={
                    "pattern": "hello",
                    "ignore_case": True
                }
            )

            assert result.is_error is False
            assert "Hello" in result.content or "WORLD" in result.content
        finally:
            self.tearDown()

    @pytest.mark.asyncio
    async def test_no_matches_found(self):
        """Test when no matches are found."""
        self.setUp()
        try:
            (Path(self.temp_dir) / "test.txt").write_text("Hello World")

            tool = GrepTool(working_dir=self.temp_dir)
            result = await tool.execute(
                tool_call_id="call_123",
                arguments={"pattern": "nonexistent"}
            )

            assert result.is_error is False
            assert "No matches found" in result.content or "0" in result.content
        finally:
            self.tearDown()

    @pytest.mark.asyncio
    async def test_max_results_limit(self):
        """Test that max results limit is respected."""
        self.setUp()
        try:
            # Create file with many matching lines
            content = "\n".join([f"match line {i}" for i in range(200)])
            (Path(self.temp_dir) / "test.txt").write_text(content)

            tool = GrepTool(working_dir=self.temp_dir)
            result = await tool.execute(
                tool_call_id="call_123",
                arguments={
                    "pattern": "match",
                    "max_results": 10
                }
            )

            # Should find at most 10 results
            match_count = result.content.count("match line")
            assert match_count <= 10
        finally:
            self.tearDown()


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



"""Tests for the extension system."""

import os
import tempfile
from pathlib import Path

import pytest

from agenix.extensions import (CommandDefinition, EventType, ExtensionContext,
                               ExtensionRunner, SessionStartEvent,
                               ToolCallEvent, ToolDefinition,
                               discover_and_load_extensions, load_extension)


@pytest.fixture
def temp_extension_dir():
    """Create a temporary directory for extensions."""
    with tempfile.TemporaryDirectory() as tmpdir:
        yield tmpdir


@pytest.fixture
def mock_agent():
    """Create a mock agent for testing."""
    class MockAgent:
        def __init__(self):
            self.messages = []
    return MockAgent()


@pytest.fixture
def extension_context(mock_agent):
    """Create an extension context."""
    return ExtensionContext(
        agent=mock_agent,
        cwd="/tmp",
        tools=[]
    )


class TestExtensionLoader:
    """Test extension loading."""

    @pytest.mark.asyncio
    async def test_load_simple_extension(self, temp_extension_dir):
        """Test loading a simple extension."""
        # Create a simple extension
        ext_file = os.path.join(temp_extension_dir, "test_ext.py")
        with open(ext_file, "w") as f:
            f.write("""
async def setup(agenix):
    from agenix.extensions import ToolDefinition

    async def my_tool(params, ctx):
        return "test result"

    agenix.register_tool(ToolDefinition(
        name="test_tool",
        description="A test tool",
        parameters={"type": "object"},
        execute=my_tool
    ))
""")

        # Load the extension
        extension = await load_extension(ext_file)

        # Verify
        assert extension is not None
        assert extension.name == "test_ext"
        assert "test_tool" in extension.tools

    @pytest.mark.asyncio
    async def test_discover_extensions(self, temp_extension_dir):
        """Test extension discovery."""
        # Create .agenix/extensions directory structure
        project_ext_dir = os.path.join(
            temp_extension_dir, ".agenix", "extensions")
        os.makedirs(project_ext_dir)

        # Create multiple extensions
        for i in range(3):
            ext_file = os.path.join(project_ext_dir, f"ext{i}.py")
            with open(ext_file, "w") as f:
                f.write(f"""
async def setup(agenix):
    pass
""")

        # Discover extensions
        extensions = await discover_and_load_extensions(
            cwd=temp_extension_dir,
            agenix_dir="/nonexistent"  # Don't use real global dir
        )

        # Should find 3 extensions from project dir
        assert len(extensions) == 3

    @pytest.mark.asyncio
    async def test_extension_with_event_handler(self, temp_extension_dir):
        """Test extension with event handlers."""
        ext_file = os.path.join(temp_extension_dir, "events.py")
        with open(ext_file, "w") as f:
            f.write("""
from agenix.extensions import EventType

async def setup(agenix):
    @agenix.on(EventType.SESSION_START)
    async def on_start(event, ctx):
        pass
""")

        extension = await load_extension(ext_file)

        assert extension is not None
        assert EventType.SESSION_START in extension.handlers
        assert len(extension.handlers[EventType.SESSION_START]) == 1

    @pytest.mark.asyncio
    async def test_extension_with_command(self, temp_extension_dir):
        """Test extension with command."""
        ext_file = os.path.join(temp_extension_dir, "commands.py")
        with open(ext_file, "w") as f:
            f.write("""
from agenix.extensions import CommandDefinition

async def setup(agenix):
    async def stats_handler(ctx, args):
        pass

    agenix.register_command(CommandDefinition(
        name="stats",
        description="Show stats",
        handler=stats_handler
    ))
""")

        extension = await load_extension(ext_file)

        assert extension is not None
        assert "stats" in extension.commands


class TestExtensionRunner:
    """Test extension runner."""

    @pytest.mark.asyncio
    async def test_emit_event(self, temp_extension_dir, extension_context):
        """Test event emission."""
        # Create extension with event handler
        ext_file = os.path.join(temp_extension_dir, "test.py")
        with open(ext_file, "w") as f:
            f.write("""
from agenix.extensions import EventType

events_received = []

async def setup(agenix):
    @agenix.on(EventType.SESSION_START)
    async def on_start(event, ctx):
        events_received.append(event.type)
""")

        extensions = [await load_extension(ext_file)]
        runner = ExtensionRunner(extensions, extension_context)

        # Emit event
        await runner.emit(SessionStartEvent())

        # Can't easily verify due to module isolation, but test runs without error

    @pytest.mark.asyncio
    async def test_get_tools(self, temp_extension_dir, extension_context):
        """Test getting registered tools."""
        ext_file = os.path.join(temp_extension_dir, "tools.py")
        with open(ext_file, "w") as f:
            f.write("""
from agenix.extensions import ToolDefinition

async def setup(agenix):
    async def tool1(params, ctx):
        return "result1"

    async def tool2(params, ctx):
        return "result2"

    agenix.register_tool(ToolDefinition(
        name="tool1",
        description="Tool 1",
        parameters={"type": "object"},
        execute=tool1
    ))

    agenix.register_tool(ToolDefinition(
        name="tool2",
        description="Tool 2",
        parameters={"type": "object"},
        execute=tool2
    ))
""")

        extensions = [await load_extension(ext_file)]
        runner = ExtensionRunner(extensions, extension_context)

        tools = runner.get_tools()
        assert len(tools) == 2
        assert "tool1" in tools
        assert "tool2" in tools

    @pytest.mark.asyncio
    async def test_execute_command(self, temp_extension_dir, extension_context):
        """Test command execution."""
        ext_file = os.path.join(temp_extension_dir, "cmd.py")
        with open(ext_file, "w") as f:
            f.write("""
from agenix.extensions import CommandDefinition

executed = []

async def setup(agenix):
    async def my_cmd(ctx, args):
        executed.append(args)

    agenix.register_command(CommandDefinition(
        name="mycmd",
        description="My command",
        handler=my_cmd
    ))
""")

        extensions = [await load_extension(ext_file)]
        runner = ExtensionRunner(extensions, extension_context)

        # Execute command
        result = await runner.execute_command("mycmd", "test args")
        assert result is True  # Command was found

        # Unknown command
        result = await runner.execute_command("unknown", "args")
        assert result is False


class TestExtensionContext:
    """Test extension context."""

    def test_context_properties(self, mock_agent):
        """Test context provides access to agent state."""
        ctx = ExtensionContext(
            agent=mock_agent,
            cwd="/test/dir",
            tools=[]
        )

        assert ctx.agent == mock_agent
        assert ctx.cwd == "/test/dir"
        assert ctx.messages == []

    def test_notify(self, mock_agent, capsys):
        """Test notification output."""
        ctx = ExtensionContext(
            agent=mock_agent,
            cwd="/tmp",
            tools=[]
        )

        ctx.notify("Test message", "info")
        captured = capsys.readouterr()
        assert "Test message" in captured.out


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



"""Test suite for tools.write module.

This module contains unit tests for the write file tool.
"""

import shutil
import tempfile
from pathlib import Path

import pytest

from agenix.tools.write import WriteTool


class TestWriteTool:
    """Test cases for WriteTool class."""

    def setUp(self):
        """Set up test fixtures."""
        self.temp_dir = tempfile.mkdtemp()

    def tearDown(self):
        """Clean up test fixtures."""
        if hasattr(self, 'temp_dir'):
            shutil.rmtree(self.temp_dir)

    def test_create_write_tool(self):
        """Test creating a write tool instance."""
        tool = WriteTool(working_dir=".")

        assert tool.name == "write"
        assert "Write" in tool.description
        assert "file_path" in tool.parameters["properties"]
        assert "content" in tool.parameters["properties"]

    @pytest.mark.asyncio
    async def test_write_simple_file(self):
        """Test writing a simple text file."""
        self.setUp()
        try:
            tool = WriteTool(working_dir=self.temp_dir)
            result = await tool.execute(
                tool_call_id="call_123",
                arguments={
                    "file_path": "test.txt",
                    "content": "Hello World"
                }
            )

            assert result.is_error is False
            assert "Successfully wrote" in result.content

            # Verify file was created
            test_file = Path(self.temp_dir) / "test.txt"
            assert test_file.exists()
            assert test_file.read_text() == "Hello World"
        finally:
            self.tearDown()

    @pytest.mark.asyncio
    async def test_write_multiline_file(self):
        """Test writing a file with multiple lines."""
        self.setUp()
        try:
            content = "Line 1\nLine 2\nLine 3"

            tool = WriteTool(working_dir=self.temp_dir)
            result = await tool.execute(
                tool_call_id="call_123",
                arguments={
                    "file_path": "multiline.txt",
                    "content": content
                }
            )

            assert result.is_error is False

            # Verify content
            test_file = Path(self.temp_dir) / "multiline.txt"
            assert test_file.read_text() == content
        finally:
            self.tearDown()

    @pytest.mark.asyncio
    async def test_write_creates_parent_directories(self):
        """Test that write creates parent directories if needed."""
        self.setUp()
        try:
            tool = WriteTool(working_dir=self.temp_dir)
            result = await tool.execute(
                tool_call_id="call_123",
                arguments={
                    "file_path": "subdir/nested/test.txt",
                    "content": "Nested file"
                }
            )

            assert result.is_error is False

            # Verify nested file was created
            test_file = Path(self.temp_dir) / "subdir" / "nested" / "test.txt"
            assert test_file.exists()
            assert test_file.read_text() == "Nested file"
        finally:
            self.tearDown()

    @pytest.mark.asyncio
    async def test_write_overwrites_existing_file(self):
        """Test that write overwrites existing files."""
        self.setUp()
        try:
            test_file = Path(self.temp_dir) / "overwrite.txt"
            test_file.write_text("Original content")

            tool = WriteTool(working_dir=self.temp_dir)
            result = await tool.execute(
                tool_call_id="call_123",
                arguments={
                    "file_path": "overwrite.txt",
                    "content": "New content"
                }
            )

            assert result.is_error is False
            assert test_file.read_text() == "New content"
        finally:
            self.tearDown()

    @pytest.mark.asyncio
    async def test_write_with_absolute_path(self):
        """Test writing file with absolute path."""
        self.setUp()
        try:
            absolute_path = Path(self.temp_dir) / "absolute.txt"

            tool = WriteTool(working_dir=".")
            result = await tool.execute(
                tool_call_id="call_123",
                arguments={
                    "file_path": str(absolute_path),
                    "content": "Absolute path test"
                }
            )

            assert result.is_error is False
            assert absolute_path.exists()
        finally:
            self.tearDown()


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



"""Test suite for core.messages module.

This module contains unit tests for message types and event classes.
"""

from datetime import datetime

import pytest

from agenix.core.messages import (AgentEndEvent, AgentStartEvent,
                                  AssistantMessage, ImageContent,
                                  MessageStartEvent, SystemMessage,
                                  TextContent, ToolCall,
                                  ToolExecutionStartEvent, ToolResultMessage,
                                  TurnStartEvent, Usage, UserMessage)


class TestUserMessage:
    """Test cases for UserMessage class."""

    def test_create_user_message(self):
        """Test creating a basic user message."""
        msg = UserMessage(content="Hello")

        assert msg.role == "user"
        assert msg.content == "Hello"
        assert isinstance(msg.timestamp, float)

    def test_user_message_with_content_list(self):
        """Test creating user message with mixed content."""
        text = TextContent(text="Hello")
        image = ImageContent(source={"type": "base64", "data": "..."})

        msg = UserMessage(content=[text, image])

        assert msg.role == "user"
        assert len(msg.content) == 2
        assert isinstance(msg.content[0], TextContent)
        assert isinstance(msg.content[1], ImageContent)


class TestAssistantMessage:
    """Test cases for AssistantMessage class."""

    def test_create_assistant_message(self):
        """Test creating a basic assistant message."""
        msg = AssistantMessage(
            content="Response",
            model="gpt-4o"
        )

        assert msg.role == "assistant"
        assert msg.content == "Response"
        assert msg.model == "gpt-4o"
        assert msg.tool_calls == []

    def test_assistant_message_with_tool_calls(self):
        """Test creating assistant message with tool calls."""
        tool_call = ToolCall(
            id="call_123",
            name="bash",
            arguments={"command": "ls"}
        )

        msg = AssistantMessage(
            content=[TextContent(text="Running command...")],
            tool_calls=[tool_call],
            model="gpt-4o"
        )

        assert len(msg.tool_calls) == 1
        assert msg.tool_calls[0].name == "bash"
        assert msg.tool_calls[0].arguments["command"] == "ls"

    def test_assistant_message_with_usage(self):
        """Test creating assistant message with usage stats."""
        usage = Usage(
            input_tokens=100,
            output_tokens=50,
            total_cost=0.005
        )

        msg = AssistantMessage(
            content="Response",
            model="gpt-4o",
            usage=usage
        )

        assert msg.usage.input_tokens == 100
        assert msg.usage.output_tokens == 50
        assert msg.usage.total_cost == 0.005


class TestToolResultMessage:
    """Test cases for ToolResultMessage class."""

    def test_create_tool_result_message(self):
        """Test creating a tool result message."""
        msg = ToolResultMessage(
            tool_call_id="call_123",
            name="bash",
            content="output here"
        )

        assert msg.role == "tool"
        assert msg.tool_call_id == "call_123"
        assert msg.name == "bash"
        assert msg.content == "output here"
        assert msg.is_error is False

    def test_create_tool_error_message(self):
        """Test creating a tool error message."""
        msg = ToolResultMessage(
            tool_call_id="call_123",
            name="bash",
            content="Error: command not found",
            is_error=True
        )

        assert msg.is_error is True


class TestTextContent:
    """Test cases for TextContent class."""

    def test_create_text_content(self):
        """Test creating text content."""
        content = TextContent(text="Hello world")

        assert content.type == "text"
        assert content.text == "Hello world"


class TestImageContent:
    """Test cases for ImageContent class."""

    def test_create_image_content(self):
        """Test creating image content."""
        source = {
            "type": "base64",
            "media_type": "image/png",
            "data": "iVBORw0KGgoAAAANS..."
        }

        content = ImageContent(source=source)

        assert content.type == "image"
        assert content.source["type"] == "base64"
        assert content.source["media_type"] == "image/png"


class TestToolCall:
    """Test cases for ToolCall class."""

    def test_create_tool_call(self):
        """Test creating a tool call."""
        tool_call = ToolCall(
            id="call_abc123",
            name="read",
            arguments={"file_path": "test.py"}
        )

        assert tool_call.id == "call_abc123"
        assert tool_call.name == "read"
        assert tool_call.arguments["file_path"] == "test.py"


class TestUsage:
    """Test cases for Usage class."""

    def test_create_usage(self):
        """Test creating usage stats."""
        usage = Usage(
            input_tokens=100,
            output_tokens=50,
            cache_read_tokens=20,
            cache_write_tokens=10,
            total_cost=0.005
        )

        assert usage.input_tokens == 100
        assert usage.output_tokens == 50
        assert usage.cache_read_tokens == 20
        assert usage.cache_write_tokens == 10
        assert usage.total_cost == 0.005


class TestEvents:
    """Test cases for event classes."""

    def test_agent_start_event(self):
        """Test creating agent start event."""
        event = AgentStartEvent()

        assert event.type == "agent_start"
        assert isinstance(event.timestamp, float)

    def test_agent_end_event(self):
        """Test creating agent end event."""
        msg = UserMessage(content="Hello")
        event = AgentEndEvent(messages=[msg])

        assert event.type == "agent_end"
        assert len(event.messages) == 1

    def test_turn_start_event(self):
        """Test creating turn start event."""
        event = TurnStartEvent()

        assert event.type == "turn_start"

    def test_message_start_event(self):
        """Test creating message start event."""
        msg = AssistantMessage(content="Test", model="gpt-4o")
        event = MessageStartEvent(message=msg)

        assert event.type == "message_start"
        assert event.message.content == "Test"

    def test_tool_execution_start_event(self):
        """Test creating tool execution start event."""
        event = ToolExecutionStartEvent(
            tool_call_id="call_123",
            tool_name="bash",
            args={"command": "ls"}
        )

        assert event.type == "tool_execution_start"
        assert event.tool_name == "bash"
        assert event.args["command"] == "ls"


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



"""Test suite for tools.base module.

This module contains unit tests for the base Tool interface.
"""

import pytest

from agenix.core.messages import TextContent
from agenix.tools.base import Tool, ToolResult


class DummyTool(Tool):
    """A dummy tool for testing purposes."""

    async def execute(self, tool_call_id, arguments, on_update=None):
        """Execute the dummy tool."""
        return ToolResult(content="dummy result")


class TestToolResult:
    """Test cases for ToolResult dataclass."""

    def test_create_tool_result(self):
        """Test creating a basic tool result."""
        result = ToolResult(content="Test output")

        assert result.content == "Test output"
        assert result.details is None
        assert result.is_error is False

    def test_create_tool_result_with_details(self):
        """Test creating tool result with details."""
        result = ToolResult(
            content="Output",
            details={"lines": 10, "path": "/test"},
            is_error=False
        )

        assert result.details["lines"] == 10
        assert result.details["path"] == "/test"

    def test_create_error_result(self):
        """Test creating an error result."""
        result = ToolResult(
            content="Error message",
            is_error=True
        )

        assert result.is_error is True


class TestTool:
    """Test cases for Tool base class."""

    def test_create_tool(self):
        """Test creating a tool instance."""
        tool = DummyTool(
            name="test",
            description="A test tool",
            parameters={"type": "object"}
        )

        assert tool.name == "test"
        assert tool.description == "A test tool"
        assert tool.parameters["type"] == "object"

    def test_to_dict(self):
        """Test converting tool to dictionary."""
        tool = DummyTool(
            name="test",
            description="A test tool",
            parameters={
                "type": "object",
                "properties": {
                    "arg1": {"type": "string"}
                }
            }
        )

        tool_dict = tool.to_dict()

        assert tool_dict["name"] == "test"
        assert tool_dict["description"] == "A test tool"
        assert "properties" in tool_dict["parameters"]

    def test_validate_arguments_success(self):
        """Test validating correct arguments."""
        tool = DummyTool(
            name="test",
            description="Test",
            parameters={
                "type": "object",
                "required": ["arg1", "arg2"]
            }
        )

        # Should not raise
        tool.validate_arguments({"arg1": "value1", "arg2": "value2"})

    def test_validate_arguments_missing_required(self):
        """Test validating arguments with missing required field."""
        tool = DummyTool(
            name="test",
            description="Test",
            parameters={
                "type": "object",
                "required": ["arg1", "arg2"]
            }
        )

        with pytest.raises(ValueError, match="Missing required argument"):
            tool.validate_arguments({"arg1": "value1"})

    @pytest.mark.asyncio
    async def test_execute_tool(self):
        """Test executing a tool."""
        tool = DummyTool(
            name="test",
            description="Test",
            parameters={}
        )

        result = await tool.execute(
            tool_call_id="call_123",
            arguments={}
        )

        assert result.content == "dummy result"
        assert result.is_error is False

    @pytest.mark.asyncio
    async def test_execute_with_callback(self):
        """Test executing tool with progress callback."""
        updates = []

        def on_update(msg):
            updates.append(msg)

        tool = DummyTool(
            name="test",
            description="Test",
            parameters={}
        )

        result = await tool.execute(
            tool_call_id="call_123",
            arguments={},
            on_update=on_update
        )

        assert result is not None


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



"""Test suite for tools.bash module.

This module contains unit tests for the bash command execution tool.
"""

import pytest

from agenix.tools.bash import BashTool


class TestBashTool:
    """Test cases for BashTool class."""

    def test_create_bash_tool(self):
        """Test creating a bash tool instance."""
        tool = BashTool(working_dir=".", timeout=60)

        assert tool.name == "bash"
        assert "bash" in tool.description.lower()
        assert "command" in tool.parameters["properties"]

    @pytest.mark.asyncio
    async def test_execute_simple_command(self):
        """Test executing a simple command."""
        tool = BashTool(working_dir=".")

        result = await tool.execute(
            tool_call_id="call_123",
            arguments={"command": "echo hello"}
        )

        assert result.is_error is False
        assert "hello" in result.content.lower()
        assert result.details["exit_code"] == 0

    @pytest.mark.asyncio
    async def test_execute_command_with_output(self):
        """Test command with stdout output."""
        tool = BashTool(working_dir=".")

        result = await tool.execute(
            tool_call_id="call_123",
            arguments={"command": "echo 'test output'"}
        )

        assert "test output" in result.content

    @pytest.mark.asyncio
    async def test_execute_failing_command(self):
        """Test command that fails."""
        tool = BashTool(working_dir=".")

        result = await tool.execute(
            tool_call_id="call_123",
            arguments={"command": "exit 1"}
        )

        assert result.is_error is True
        assert result.details["exit_code"] == 1

    @pytest.mark.asyncio
    async def test_execute_nonexistent_command(self):
        """Test executing a command that doesn't exist."""
        tool = BashTool(working_dir=".")

        result = await tool.execute(
            tool_call_id="call_123",
            arguments={"command": "nonexistent_command_xyz"}
        )

        # Should either be error or non-zero exit code
        assert result.is_error is True or result.details.get("exit_code") != 0

    @pytest.mark.asyncio
    async def test_working_directory(self):
        """Test that command executes in correct working directory."""
        import os
        import tempfile

        temp_dir = tempfile.mkdtemp()
        try:
            tool = BashTool(working_dir=temp_dir)

            result = await tool.execute(
                tool_call_id="call_123",
                arguments={"command": "pwd"}
            )

            assert temp_dir in result.content
        finally:
            os.rmdir(temp_dir)

    @pytest.mark.asyncio
    async def test_command_with_stderr(self):
        """Test command that writes to stderr."""
        tool = BashTool(working_dir=".")

        result = await tool.execute(
            tool_call_id="call_123",
            arguments={"command": "echo 'error' >&2"}
        )

        # Should capture stderr
        assert "Stderr" in result.content or "error" in result.content


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



"""Setup script for agenix."""

from setuptools import setup, find_packages
from pathlib import Path

# Read README for long description
readme_file = Path(__file__).parent / "README.md"
long_description = readme_file.read_text(encoding="utf-8") if readme_file.exists() else ""

# Dependencies
requirements = [
    "openai>=1.12.0",
    "anthropic>=0.18.0",
    "rich>=13.7.0",
    "pyyaml>=6.0",
    "prompt_toolkit>=3.0.0",  # Better terminal input with unicode support
]

dev_requirements = [
    "pytest>=7.4.0",
    "pytest-asyncio>=0.21.0",
    "pytest-cov>=4.1.0",
]

docs_requirements = [
    "sphinx>=8.0.0",
    "sphinx-rtd-theme>=3.0.0",
    "myst-parser>=5.0.0",
]

setup(
    name="agenix",
    version="0.0.1",
    author="ZHANG Zhi",
    author_email="tczhangzhi@gmail.com",
    description="A lightweight AI coding agent with file operations and shell execution capabilities",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/tczhangzhi/agenix",
    project_urls={
        "Bug Tracker": "https://github.com/tczhangzhi/agenix/issues",
        "Source Code": "https://github.com/tczhangzhi/agenix",
        "Changelog": "https://github.com/tczhangzhi/agenix/blob/main/CHANGELOG.md",
    },
    packages=find_packages(exclude=["tests", "tests.*", "docs", "examples"]),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Operating System :: OS Independent",
        "Environment :: Console",
    ],
    python_requires=">=3.8",
    install_requires=requirements,
    extras_require={
        "dev": dev_requirements,
        "docs": docs_requirements,
    },
    entry_points={
        "console_scripts": [
            "agenix=agenix.cli:main",
        ],
    },
    include_package_data=True,
    zip_safe=False,
    keywords="ai agent coding llm openai anthropic claude automation cli assistant gpt-4 agenix",
)



"""Test suite for tools.read module.

This module contains unit tests for the read file tool.
"""

import os
import tempfile
from pathlib import Path

import pytest

from agenix.tools.read import ReadTool


class TestReadTool:
    """Test cases for ReadTool class."""

    def setUp(self):
        """Set up test fixtures."""
        self.temp_dir = tempfile.mkdtemp()

    def tearDown(self):
        """Clean up test fixtures."""
        import shutil
        if hasattr(self, 'temp_dir'):
            shutil.rmtree(self.temp_dir)

    def test_create_read_tool(self):
        """Test creating a read tool instance."""
        tool = ReadTool(working_dir=".")

        assert tool.name == "read"
        assert "Read" in tool.description
        assert "file_path" in tool.parameters["properties"]

    @pytest.mark.asyncio
    async def test_read_simple_file(self):
        """Test reading a simple text file."""
        self.setUp()
        try:
            # Create test file
            test_file = Path(self.temp_dir) / "test.txt"
            test_file.write_text("Hello\nWorld\n")

            tool = ReadTool(working_dir=self.temp_dir)
            result = await tool.execute(
                tool_call_id="call_123",
                arguments={"file_path": "test.txt"}
            )

            assert result.is_error is False
            assert "Hello" in result.content
            assert "World" in result.content
        finally:
            self.tearDown()

    @pytest.mark.asyncio
    async def test_read_with_line_numbers(self):
        """Test that read output includes line numbers."""
        self.setUp()
        try:
            test_file = Path(self.temp_dir) / "test.txt"
            test_file.write_text("Line 1\nLine 2\nLine 3\n")

            tool = ReadTool(working_dir=self.temp_dir)
            result = await tool.execute(
                tool_call_id="call_123",
                arguments={"file_path": "test.txt"}
            )

            # Should have line numbers (format: "     1\t")
            assert "1\t" in result.content or "Line 1" in result.content
        finally:
            self.tearDown()

    @pytest.mark.asyncio
    async def test_read_with_offset_and_limit(self):
        """Test reading file with offset and limit."""
        self.setUp()
        try:
            test_file = Path(self.temp_dir) / "test.txt"
            lines = [f"Line {i}\n" for i in range(1, 11)]
            test_file.write_text("".join(lines))

            tool = ReadTool(working_dir=self.temp_dir)
            result = await tool.execute(
                tool_call_id="call_123",
                arguments={
                    "file_path": "test.txt",
                    "offset": 3,
                    "limit": 3
                }
            )

            assert result.is_error is False
            # Should show truncation info
            assert "Showing lines" in result.content or "Line 3" in result.content
        finally:
            self.tearDown()

    @pytest.mark.asyncio
    async def test_read_nonexistent_file(self):
        """Test reading a file that doesn't exist."""
        tool = ReadTool(working_dir=self.temp_dir if hasattr(
            self, 'temp_dir') else ".")

        result = await tool.execute(
            tool_call_id="call_123",
            arguments={"file_path": "nonexistent.txt"}
        )

        assert result.is_error is True
        assert "not found" in result.content.lower()

    @pytest.mark.asyncio
    async def test_read_absolute_path(self):
        """Test reading file with absolute path."""
        self.setUp()
        try:
            test_file = Path(self.temp_dir) / "test.txt"
            test_file.write_text("Absolute path test")

            tool = ReadTool(working_dir=".")
            result = await tool.execute(
                tool_call_id="call_123",
                arguments={"file_path": str(test_file)}
            )

            assert result.is_error is False
            assert "Absolute path test" in result.content
        finally:
            self.tearDown()


if __name__ == "__main__":
    pytest.main([__file__, "-v"])



"""Programmatic SDK for using agenix as a library.

Example usage:

```python
import asyncio
from agenix import create_session

async def main():
    # Create a session
    session = await create_session(
        api_key="your-api-key",
        model="gpt-4o",
        working_dir="/path/to/project"
    )

    # Send a prompt
    response = await session.prompt("What files are in the current directory?")
    print(response)

    # Continue conversation
    response = await session.prompt("Can you read the README?")
    print(response)

    # Get conversation history
    messages = session.get_messages()
    print(f"Conversation has {len(messages)} messages")

if __name__ == "__main__":
    asyncio.run(main())
```
"""

from .extensions import (CommandDefinition, EventType, ExtensionAPI,
                         ExtensionSetup, ToolDefinition)
import asyncio
import os
from pathlib import Path
from typing import Any, Dict, List, Optional

from .core.agent import Agent, AgentConfig
from .core.messages import Message
from .extensions import (ExtensionContext, ExtensionRunner, SessionEndEvent,
                         SessionStartEvent, discover_and_load_extensions)
from .tools.bash import BashTool
from .tools.edit import EditTool
from .tools.grep import GrepTool
from .tools.read import ReadTool
from .tools.write import WriteTool


class AgentSession:
    """High-level agent session for programmatic usage.

    Wraps the low-level Agent class with a simpler API.
    """

    def __init__(
        self,
        agent: Agent,
        tools: List[Any],
        extension_runner: Optional[ExtensionRunner] = None,
        working_dir: str = "."
    ):
        self.agent = agent
        self.tools = tools
        self.extension_runner = extension_runner
        self.working_dir = working_dir
        self._started = False

    async def _ensure_started(self) -> None:
        """Ensure session has been started."""
        if not self._started:
            if self.extension_runner:
                await self.extension_runner.emit(SessionStartEvent())
            self._started = True

    async def prompt(self, message: str) -> str:
        """Send a prompt to the agent and return the response.

        Args:
            message: User message/prompt

        Returns:
            Agent's text response
        """
        await self._ensure_started()

        # Collect the response
        response_parts = []

        async for event in self.agent.prompt(message):
            # Collect text from message update events
            from .core.messages import MessageUpdateEvent
            if isinstance(event, MessageUpdateEvent):
                response_parts.append(event.delta)

        return "".join(response_parts)

    def get_messages(self) -> List[Message]:
        """Get conversation history.

        Returns:
            List of messages in the conversation
        """
        return self.agent.messages

    def clear_messages(self) -> None:
        """Clear conversation history."""
        self.agent.clear_messages()

    async def close(self) -> None:
        """Close the session and cleanup."""
        if self.extension_runner:
            await self.extension_runner.emit(SessionEndEvent())


async def create_session(
    api_key: Optional[str] = None,
    base_url: Optional[str] = None,
    model: str = "gpt-4o",
    system_prompt: Optional[str] = None,
    working_dir: str = ".",
    max_turns: int = 10,
    enable_extensions: bool = True
) -> AgentSession:
    """Create an agent session for programmatic usage.

    Args:
        api_key: API key for the LLM provider (or set OPENAI_API_KEY env var)
        base_url: Optional API base URL
        model: Model to use (default: gpt-4o)
        system_prompt: Optional custom system prompt
        working_dir: Working directory for file operations
        max_turns: Maximum conversation turns per prompt
        enable_extensions: Whether to load extensions

    Returns:
        AgentSession instance

    Example:
        >>> import asyncio
        >>> from agenix import create_session
        >>>
        >>> async def main():
        ...     session = await create_session(
        ...         api_key="your-key",
        ...         model="gpt-4o"
        ...     )
        ...     response = await session.prompt("Hello!")
        ...     print(response)
        ...
        >>> asyncio.run(main())
    """
    # Get API key
    if not api_key:
        api_key = os.getenv("OPENAI_API_KEY")
    if not api_key:
        raise ValueError(
            "API key required. Set OPENAI_API_KEY environment variable "
            "or pass api_key parameter."
        )

    # Resolve working directory
    working_dir = os.path.abspath(working_dir)

    # Setup tools
    tools = [
        ReadTool(working_dir=working_dir),
        WriteTool(working_dir=working_dir),
        EditTool(working_dir=working_dir),
        BashTool(working_dir=working_dir),
        GrepTool(working_dir=working_dir),
    ]

    # Load extensions if enabled
    extension_runner = None
    if enable_extensions:
        extensions = await discover_and_load_extensions(cwd=working_dir)
        if extensions:
            # Create extension context
            # Note: We need an agent instance, so we'll create it first
            # and then set up extensions
            pass

    # Get default system prompt if not provided
    if not system_prompt:
        from .cli import get_default_system_prompt
        system_prompt = get_default_system_prompt(tools)

    # Create agent config
    config = AgentConfig(
        model=model,
        api_key=api_key,
        base_url=base_url,
        system_prompt=system_prompt,
        max_turns=max_turns
    )

    # Create agent
    agent = Agent(config=config, tools=tools)

    # Setup extension runner if extensions were loaded
    if enable_extensions:
        extensions = await discover_and_load_extensions(cwd=working_dir)
        if extensions:
            ext_context = ExtensionContext(
                agent=agent,
                cwd=working_dir,
                tools=tools
            )
            extension_runner = ExtensionRunner(extensions, ext_context)

    # Create session
    session = AgentSession(
        agent=agent,
        tools=tools,
        extension_runner=extension_runner,
        working_dir=working_dir
    )

    return session


# Re-export key types for convenience

__all__ = [
    # Main API
    "create_session",
    "AgentSession",

    # Extension types (for custom extensions)
    "ToolDefinition",
    "CommandDefinition",
    "ExtensionAPI",
    "ExtensionSetup",
    "EventType",
]
